@@ -28,7 +28,7 @@ Continue <- TRUE
 while(Continue==TRUE)
 {
 #Using Random.Predictors=FALSE here to get an estimate for sample size.
-CI.0 <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.0, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.0, p=p, Random.Predictors=FALSE, ...)
+CI.0 <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.0, p=p), conf.level=NULL, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.0, p=p, Random.Predictors=FALSE, ...)
 Continue <- sum(is.na(CI.0))>0
 N.0 <- N.0 + 1
 }
@@ -36,9 +36,8 @@ N.0 <- N.0 + 1
 if(which.width=="Full")
 {
 
-
 N.1 <- N.0 + 1
-CI.1 <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.1, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.1, p=p, Random.Predictors=FALSE)
+CI.1 <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.1, p=p), conf.level=NULL, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.1, p=p, Random.Predictors=FALSE)
 w.F <- CI.1$Upper.Conf.Limit.R2 - CI.1$Lower.Conf.Limit.R2
 Diff <- w.F - width
 while(Diff > Tol | is.na(Diff))
@@ -47,6 +46,7 @@ N.1 <- N.1 + 1
 CI.1 <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.1, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.1, p=p, Random.Predictors=FALSE)
 w.F <- CI.1$Upper.Conf.Limit.R2 - CI.1$Lower.Conf.Limit.R2
 Diff <- w.F - width
+
 }
 
 if(Random.Predictors==TRUE)
@@ -66,33 +66,37 @@ Diff <- w.F - width
 
 Result.Full <- list(Required.Sample.Size=N.1, Expected.Width=w.F)
 
+# All above from this point is the standard procedure.
 if(!is.null(degree.of.certainty)) 
 {
-Conf.Limit.Desired.Certainty.Upper <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.1, p=p), alpha.lower=0, alpha.upper=1-degree.of.certainty, N=N.1, p=p, Random.Predictors=Random.Predictors)$Upper.Conf.Limit.R2
 Conf.Limit.Desired.Certainty.Lower <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.1, p=p), alpha.lower=1-degree.of.certainty, alpha.upper=0, N=N.1, p=p, Random.Predictors=Random.Predictors)$Lower.Conf.Limit.R2
+Conf.Limit.Desired.Certainty.Upper <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.1, p=p), alpha.lower=0, alpha.upper=1-degree.of.certainty, N=N.1, p=p, Random.Predictors=Random.Predictors)$Upper.Conf.Limit.R2
 
-N.2 <- N.1 + 1
-CI.2 <- ci.R2(R2=Expected.R2(Population.R2=Conf.Limit.Desired.Certainty.Upper, N=N.2, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.2, p=p, Random.Predictors=Random.Predictors)
+N.2 <- N.1
+CI.2 <- ci.R2(R2=Conf.Limit.Desired.Certainty.Upper, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.2, p=p, Random.Predictors=Random.Predictors)
 w.F <- CI.2$Upper.Conf.Limit.R2 - CI.2$Lower.Conf.Limit.R2
 Diff <- w.F - width
 while(Diff > Tol | is.na(Diff))
 {
 N.2 <- N.2 + 1
-CI.2 <- ci.R2(R2=Expected.R2(Population.R2=Conf.Limit.Desired.Certainty.Upper, N=N.2, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.2, p=p, Random.Predictors=Random.Predictors)
+# The new expected limit is calculated each time.
+Conf.Limit.Desired.Certainty.Upper.i <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.2, p=p), alpha.lower=0, alpha.upper=1-degree.of.certainty, N=N.2, p=p, Random.Predictors=Random.Predictors)$Upper.Conf.Limit.R2
+CI.2 <- ci.R2(R2=Conf.Limit.Desired.Certainty.Upper.i, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.2, p=p, Random.Predictors=Random.Predictors)
 w.F <- CI.2$Upper.Conf.Limit.R2 - CI.2$Lower.Conf.Limit.R2
 Diff <- w.F - width
 }
 N.Upper.Conf.Lim <- N.2
 Ex.Width.Upper <- w.F
 
-N.3 <- N.1 + 1
-CI.3 <- ci.R2(R2=Expected.R2(Population.R2=Conf.Limit.Desired.Certainty.Lower, N=N.3, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.3, p=p, Random.Predictors=Random.Predictors)
+N.3 <- N.1
+CI.3 <- ci.R2(R2=Conf.Limit.Desired.Certainty.Lower, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.3, p=p, Random.Predictors=Random.Predictors)
 w.F <- CI.3$Upper.Conf.Limit.R2 - CI.3$Lower.Conf.Limit.R2
 Diff <- w.F - width
 while(Diff > Tol | is.na(Diff))
 {
 N.3 <- N.3 + 1
-CI.3 <- ci.R2(R2=Expected.R2(Population.R2=Conf.Limit.Desired.Certainty.Lower, N=N.3, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.3, p=p, Random.Predictors=Random.Predictors)
+Conf.Limit.Desired.Certainty.Lower.i <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=N.3, p=p), alpha.lower=1-degree.of.certainty, alpha.upper=0, N=N.3, p=p, Random.Predictors=Random.Predictors)$Lower.Conf.Limit.R2
+CI.3 <- ci.R2(R2=Conf.Limit.Desired.Certainty.Lower.i, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.3, p=p, Random.Predictors=Random.Predictors)
 w.F <- CI.3$Upper.Conf.Limit.R2 - CI.3$Lower.Conf.Limit.R2
 Diff <- w.F - width
 }
@@ -109,80 +113,102 @@ if(N.Lower.Conf.Lim > N.Upper.Conf.Lim)
 Result.Full <- list(Required.Sample.Size=N.Lower.Conf.Lim, Expected.Width=Ex.Width.Lower)
 }
 
-
 # To get maximum expected CI width to ensure adequate sample size is used.
 CI.WIDTH.R2 <- function(R2, conf.level, N, p, Random.Predictors)
 {
+# Originally used expected width in the interval; however, expectation and the limits have already been found and the value stays within the interval based on expectation.
 Lims <- ci.R2(R2=Expected.R2(Population.R2=R2, N, p), conf.level=conf.level, N=N, p=p, Random.Predictors=Random.Predictors)
+
+# Considered *not* using the expected value, but it seem to work better with the expected value.
+#Lims <- ci.R2(R2=R2, conf.level=conf.level, N=N, p=p, Random.Predictors=Random.Predictors)
 Lims$Upper - Lims$Lower
 }
-Optimize.Result <- optimize(f=CI.WIDTH.R2, interval = c(Conf.Limit.Desired.Certainty.Lower, Conf.Limit.Desired.Certainty.Upper), maximum = TRUE, tol = .Machine$double.eps^0.5, N=N.1, p=p, conf.level=conf.level, Random.Predictors=Random.Predictors)$maximum
+# Based on original estimate of sample size.
+
+Optimize.Result <- optimize(f=CI.WIDTH.R2, interval = c(0, 1), maximum = TRUE, tol = .Machine$double.eps^0.5, N=N.1, p=p, conf.level=conf.level, Random.Predictors=Random.Predictors)$maximum
+
+N.Op <- NULL
+N.Op.Up <- NULL
+N.Op.Low <- NULL
 
 # The following is used if and only if the maximum confidence interval width is between the confidence limits.
 if((round(Conf.Limit.Desired.Certainty.Lower,4) < round(Optimize.Result,4)) & (round(Optimize.Result,4) < round(Conf.Limit.Desired.Certainty.Upper,4)))
 {
+# Optimize.Result is in terms of the expected value.
+Lims <- ci.R2(R2=Optimize.Result, conf.level=(1-degree.of.certainty), N=N.1, p=p, Random.Predictors=Random.Predictors)
 
-Prob.F.Max.Width <- pf(q=Rsquare2F(R2=Optimize.Result, p=p, N=N.1), df1=p, df2=N.1-p-1, ncp=Rsquare2Lambda(R2=Population.R2, N=N.1))
+Conf.Limit.Desired.Certainty.Lower <- Lims$Lower
+Conf.Limit.Desired.Certainty.Upper <- Lims$Upper
 
-low.lim.F <- qf(p=max(0, (Prob.F.Max.Width-(1-degree.of.certainty)/2)), df1=p, df2=N.1-p-1, ncp=Rsquare2Lambda(R2=Population.R2, N=N.1))
-up.lim.F <- qf(p=min((Prob.F.Max.Width+(1-degree.of.certainty)/2), 1), df1=p, df2=N.1-p-1, ncp=Rsquare2Lambda(R2=Population.R2, N=N.1))
-
-low.R2 <- F2Rsquare(F.value=low.lim.F, df.1 = p, df.2 = N.1-p-1)
-up.R2 <- F2Rsquare(F.value=up.lim.F, df.1 = p, df.2 = N.1-p-1)
-
-# Here SS is determined for the upper limit from the optimize method.
-if(up.R2>0 & up.R2<1)
+if(Conf.Limit.Desired.Certainty.Lower < Optimize.Result)
 {
-N.4 <- N.1 + 1
-CI.4 <- ci.R2(R2=Expected.R2(Population.R2=up.R2, N=N.4, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.4, p=p, Random.Predictors=Random.Predictors)
-w.F <- CI.4$Upper.Conf.Limit.R2 - CI.4$Lower.Conf.Limit.R2
+N.Op.Low <- N.1
+CI.Op.Low <- ci.R2(R2=Conf.Limit.Desired.Certainty.Lower, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.Op.Low, p=p, Random.Predictors=Random.Predictors)
+w.F <- CI.Op.Low$Upper.Conf.Limit.R2 - CI.Op.Low$Lower.Conf.Limit.R2
 Diff <- w.F - width
 while(Diff > Tol | is.na(Diff))
 {
-N.4 <- N.4 + 1
-CI.4 <- ci.R2(R2=Expected.R2(Population.R2=up.R2, N=N.4, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.4, p=p, Random.Predictors=Random.Predictors)
-w.F <- CI.4$Upper.Conf.Limit.R2 - CI.4$Lower.Conf.Limit.R2
+N.Op.Low <- N.Op.Low + 1
+CI.Op.Low <- ci.R2(R2=Conf.Limit.Desired.Certainty.Lower, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.Op.Low, p=p, Random.Predictors=Random.Predictors)
+w.F <- CI.Op.Low$Upper.Conf.Limit.R2 - CI.Op.Low$Lower.Conf.Limit.R2
 Diff <- w.F - width
 }
-n.up.R2 <- N.4
-Ex.Width.Upper.after.optim <- w.F
+w.Op.Width.Low <- w.F
+}
+if(Conf.Limit.Desired.Certainty.Lower >= Optimize.Result)
+{
+N.Op.Low <- N.1
+w.Op.Width.Low <- 1
 }
 
-# Here SS is determined for the lower limit from the optimize method.
-if(low.R2>0 & low.R2<1)
+if(Conf.Limit.Desired.Certainty.Upper > Optimize.Result)
 {
-N.5 <- N.1 + 1
-CI.5 <- ci.R2(R2=Expected.R2(Population.R2=low.R2, N=N.5, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.5, p=p, Random.Predictors=Random.Predictors)
-w.F <- CI.5$Upper.Conf.Limit.R2 - CI.5$Lower.Conf.Limit.R2
+N.Op.Up <- N.1
+CI.Op.Up <- ci.R2(R2=Conf.Limit.Desired.Certainty.Upper, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.Op.Up, p=p, Random.Predictors=Random.Predictors)
+w.F <- CI.Op.Low$Upper.Conf.Limit.R2 - CI.Op.Low$Lower.Conf.Limit.R2
 Diff <- w.F - width
 while(Diff > Tol | is.na(Diff))
 {
-N.5 <- N.5 + 1
-CI.5 <- ci.R2(R2=Expected.R2(Population.R2=low.R2, N=N.5, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.5, p=p, Random.Predictors=Random.Predictors)
-w.F <- CI.5$Upper.Conf.Limit.R2 - CI.5$Lower.Conf.Limit.R2
+N.Op.Up <- N.Op.Up + 1
+CI.Op.Up <- ci.R2(R2=Conf.Limit.Desired.Certainty.Upper, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.Op.Up, p=p, Random.Predictors=Random.Predictors)
+w.F <- CI.Op.Up$Upper.Conf.Limit.R2 - CI.Op.Up$Lower.Conf.Limit.R2
 Diff <- w.F - width
 }
-n.low.R2 <- N.5
-Ex.Width.Lower.after.optim <- w.F
+w.Op.Width.Up <- w.F
 }
-
-# To keep function from failing when an impossible low.R2 arises.
-if(low.R2==0 | low.R2==1) n.low.R2 <- N.1
-if(up.R2==0 | up.R2==1) n.up.R2 <- N.1
-
-if((n.low.R2 >= n.up.R2) & (n.low.R2 > Result.Full$Required.Sample.Size))
+if(Conf.Limit.Desired.Certainty.Upper <= Optimize.Result)
 {
-Result.Full <- list(Required.Sample.Size=n.low.R2, Expected.Width=Ex.Width.Lower.after.optim)
+N.Op.Up <- N.1
+w.Op.Width.Up <- 1
 }
 
-if((n.up.R2 > n.low.R2) & (n.up.R2 > Result.Full$Required.Sample.Size))
+
+# Determine if the sample size should be based on the maximum CI width
+if(N.1==N.Lower.Conf.Lim & N.1==N.Upper.Conf.Lim & N.1==N.Op.Up & N.1==N.Op.Low)
 {
-Result.Full <- list(Required.Sample.Size=n.up.R2, Expected.Width=Ex.Width.Upper.after.optim)
+N.Op <- N.1
+
+CI.Op<- ci.R2(R2=Optimize.Result, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.Op, p=p, Random.Predictors=Random.Predictors)
+w.F <- CI.Op$Upper.Conf.Limit.R2 - CI.Op$Lower.Conf.Limit.R2
+Diff <- w.F - width
+while(Diff > Tol | is.na(Diff))
+{
+N.Op <- N.Op + 1
+CI.Op <- ci.R2(R2=Optimize.Result, alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=N.Op, p=p, Random.Predictors=Random.Predictors)
+w.F <- CI.Op$Upper.Conf.Limit.R2 - CI.Op$Lower.Conf.Limit.R2
+Diff <- w.F - width
+}
+w.Op <- w.F
 }
 }
 
+Necessary.N <- max(N.1, N.Lower.Conf.Lim, N.Upper.Conf.Lim, N.Op.Low, N.Op.Up, N.Op, na.rm=TRUE)
+
+For.Ex.Width <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=Necessary.N, p=p), alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=Necessary.N, p=p, Random.Predictors=Random.Predictors)
+Result.Full <- list(Required.Sample.Size=Necessary.N, Expected.Width=For.Ex.Width$Upper.Conf.Limit.R2 - For.Ex.Width$Lower.Conf.Limit.R2)
 }
 
+
 # Get the expected width.
 tmp.MBESS <- ci.R2(R2=Expected.R2(Population.R2=Population.R2, N=Result.Full$Required.Sample.Size, p=p),
 alpha.lower=alpha.lower, alpha.upper=alpha.upper, N=Result.Full$Required.Sample.Size, p=p, 
